#ifndef SUMMARY_DEFINES_H
#define SUMMARY_DEFINES_H


#define META_NAME			"meta:name"
#define META_GENERATOR			"meta:generator"
#define META_CREATION_DATE 		"meta:creation-date"
#define META_PRINT_DATE			"meta:print-date"
#define META_EDITING_CYCLES		"meta:editing-cycles"
#define META_EDITING_DURATION		"meta:editing-duration"
#define META_INITIAL_CREATOR		"meta:initial-creator"
#define META_KEYWORD			"meta:keyword"
#define META_USER_DEFINED		"meta:user-defined"
#define META_DOCUMENT_STATISTIC		"meta:document-statistic"
#define META_DOC_STAT_PAGE_COUNT	"meta:page-count"
#define META_DOC_STAT_PARAGRAPH_COUNT	"meta:paragraph-count"
#define META_DOC_STAT_WORD_COUNT	"meta:word-count"
#define META_DOC_STAT_CHARACTER_COUNT	"meta:character-count"
#define META_DOC_STAT_IMAGE_COUNT	"meta:image-count"
#define META_DOC_STAT_TABLE_COUNT	"meta:table-count"
#define META_DOC_STAT_CELL_COUNT	"meta:cell-count"
#define META_DOC_STAT_OBJECT_COUNT	"meta:object-count"
#define META_DOC_STAT_OLE_OBJECT_COUNT	"meta:ole-object-count"
#define META_DOC_STAT_ROW_COUNT		"meta:row-count"
#define META_DOC_STAT_DRAW_COUNT	"meta:draw-count"
#define DC_TITLE			"dc:title"
#define DC_DATE				"dc:date"
#define DC_LANGUAGE			"dc:language"
#define DC_SUBJECT			"dc:subject"
#define DC_DESCRIPTION			"dc:description"
#define DC_CREATOR			"dc:creator"
#define BEGIN_TAG			'<'
#define END_TAG				'>'
#define SINGLE				'/'
#define XML_HEADER_BEGIN		"<?"
#define XML_HEADER_END			"?>"



#define MAX_BUFF_SIZE 5000


#endif